#include "RA8889_MCU_IF.h"
#include "delay.h"
#include <intrins.h>
   
//----------------------------------------------------------------Hardware SPI 4wireSPI-------------------------------------------------------------
#if Peripheral_HardwareSPI

#define SS_RESET   (P13 = 0)
#define SS_SET     (P13 = 1)

void HardwareSPI_Init(void)
{
	unsigned char temp = 0;
	
	P1M0 |= (1<<3);
	P1M1 &= ~(1<<3);

	// RST
	P3M0 |= (1<<7);
	P3M1 &= ~(1<<7);
	
//	temp = P_SW1;
	temp |= (1<<2);
	temp |= (1<<3);
//	P_SW1 = temp;
	
	SPSTAT = 0xc0;
	SPCTL = 0xdc;
}


void SpiWrite(unsigned char byte)
{
	SPSTAT =SPIF | WCOL;           //SCLK:P17   MISO:P16  MOSI:P15                     
	SPDAT=byte;                                        
	while (!(SPSTAT & SPIF ));                          
}

unsigned char SpiRead(void)
{
	SPSTAT=SPIF | WCOL;                                
	SPDAT=0xff;                                         
	while (!(SPSTAT & SPIF));                        
	return SPDAT;
}


void LCD_CmdWrite(unsigned char cmd)
{
	SS_RESET; 	      
	SpiWrite(0x00);
	SpiWrite(cmd);
	SS_SET;
}

void LCD_DataWrite(unsigned char dat)
{
	SS_RESET; 	      
	SpiWrite(0x80);
	SpiWrite(dat);
	SS_SET;
}

void LCD_DataWrite_Pixel(unsigned int dat)
{
	SS_RESET; 	      
	SpiWrite(0x80);
	SpiWrite(dat);
	SS_SET;
	
	SS_RESET;
	SpiWrite(0x80);
	SpiWrite(dat>>8);
	SS_SET;
}

unsigned char LCD_StatusRead(void)
{
	unsigned char temp = 0;
	SS_RESET; 	      
	SpiWrite(0x40);
	temp = SpiRead();
	SS_SET;
	return temp;
}

unsigned int LCD_DataRead(void)
{
	unsigned int temp = 0;
	SS_RESET; 	      
	SpiWrite(0xc0);
	temp = SpiRead();
	SS_SET;
	return temp;
}

#endif


//----------------------------------------------------------------Software 3 WIRE SPI-------------------------------------------------------------
#if Peripheral_Software3SPI

#define SS_RESET   (P13 = 0)
#define SS_SET     (P13 = 1)
#define SCLK  P11
#define SDIO  P12

void Software3SPI_Init(void)
{
	unsigned char temp = 0;
	
	P1M0 |= (1<<3);	//CS
	P1M1 &= ~(1<<3);

	
	P3M0 |= (1<<7); // RST
	P3M1 &= ~(1<<7);

 	P1M0 |= (1<<1);	  //SCLK
	P1M1 &= ~(1<<1);

	P1M0 |= (1<<2);	  //SDI
	P1M1 &= ~(1<<2);	
		

}


void SpiWrite(unsigned char byte)
{  	unsigned char t = 8;                           

	P1M0 |= (1<<2);	  //SDI
	P1M1 &= ~(1<<2);	
	do
	{
		SDIO = (bit)(byte & 0x80);
		byte <<= 1;
		SCLK = 0;	
		SCLK = 1;
	} while ( --t != 0 );
	SCLK = 1;
	SDIO = 1; 

	                        
}

unsigned char SpiRead(void)
{	unsigned char byte;
	unsigned char t = 8;
		
	P1M0 &= ~(1<<2);  //SDO 
	P1M1 |= (1<<2);
	do
	{
		SCLK = 0;
		byte <<= 1;
		if ( SDIO ) byte++;
		SCLK = 1;
	} while ( --t != 0 );

 	P1M0 |= (1<<2);	  
	P1M1 &= ~(1<<2);

	return byte;


}
////////////////////////////////////////////////////////////////////



void LCD_CmdWrite(unsigned char cmd)
{
	SS_RESET; 	      
	SpiWrite(0x00);
	SpiWrite(cmd);
	SS_SET;
}

void LCD_DataWrite(unsigned char dat)
{
	SS_RESET; 	      
	SpiWrite(0x80);
	SpiWrite(dat);
	SS_SET;
}

void LCD_DataWrite_Pixel(unsigned int dat)
{
	SS_RESET; 	      
	SpiWrite(0x80);
	SpiWrite(dat);
	SS_SET;
	
	SS_RESET;
	SpiWrite(0x80);
	SpiWrite(dat>>8);
	SS_SET;
}

unsigned char LCD_StatusRead(void)
{
	unsigned char temp = 0;
	SS_RESET; 	      
	SpiWrite(0x40);
	temp = SpiRead();
	SS_SET;
	return temp;
}

unsigned int LCD_DataRead(void)
{
	unsigned int temp = 0;
	SS_RESET; 	      
	SpiWrite(0xc0);
	temp = SpiRead();
	SS_SET;
	return temp;
}

#endif



//----------------------------------------------------------------Software 4 WIRE SPI-------------------------------------------------------------
#if Peripheral_Software4SPI

#define SS_RESET   (P13 = 0)
#define SS_SET     (P13 = 1)
#define SDI   P10
#define SCLK  P11
#define SDO   P12

void Software4SPI_Init(void)
{
	unsigned char temp = 0;
	
	P1M0 |= (1<<3);	//CS
	P1M1 &= ~(1<<3);

	
	P3M0 |= (1<<7); // RST
	P3M1 &= ~(1<<7);

	P1M0 |= (1<<0);	  //SDI
	P1M1 &= ~(1<<0);	
	P1M0 |= (1<<1);	  //SCLK
	P1M1 &= ~(1<<1);		
	P1M0 &= ~(1<<2);  //SDO 
	P1M1 |= (1<<2);
}


void SpiWrite(unsigned char byte)
{  	unsigned char t = 8;                           

	do
	{
		SDI = (bit)(byte & 0x80);
		byte <<= 1;
		SCLK = 0;	
		SCLK = 1;
	} while ( --t != 0 );
	SCLK = 1;
	SDI = 1; 

	                        
}

unsigned char SpiRead(void)
{	unsigned char byte;
	unsigned char t = 8;	

	do
	{
		SCLK = 0;
		byte <<= 1;
		if ( SDO ) byte++;
		SCLK = 1;
	} while ( --t != 0 );
	return byte;


}
////////////////////////////////////////////////////////////////////



void LCD_CmdWrite(unsigned char cmd)
{
	SS_RESET; 	      
	SpiWrite(0x00);
	SpiWrite(cmd);
	SS_SET;
}

void LCD_DataWrite(unsigned char dat)
{
	SS_RESET; 	      
	SpiWrite(0x80);
	SpiWrite(dat);
	SS_SET;
}

void LCD_DataWrite_Pixel(unsigned int dat)
{
	SS_RESET; 	      
	SpiWrite(0x80);
	SpiWrite(dat);
	SS_SET;
	
	SS_RESET;
	SpiWrite(0x80);
	SpiWrite(dat>>8);
	SS_SET;
}

unsigned char LCD_StatusRead(void)
{
	unsigned char temp = 0;
	SS_RESET; 	      
	SpiWrite(0x40);
	temp = SpiRead();
	SS_SET;
	return temp;
}

unsigned int LCD_DataRead(void)
{
	unsigned int temp = 0;
	SS_RESET; 	      
	SpiWrite(0xc0);
	temp = SpiRead();
	SS_SET;
	return temp;
}

#endif



//-------------------------------------------------------------8λ8080--------------------------------------------------------

#if Peripheral_8080_8

#include <intrins.h>

#define LCD_WR  P12
#define LCD_RD  P13
#define LCD_RS  P11
#define LCD_CS  P10


void delay(void)
{
	unsigned char i = 1;
	while(i--)
	{
		_nop_();
	}
}

void Peripheral_8080_8_Init(void)
{
	// 8λ
	P0M0 = 0xff;
	P0M1 = 0x00; 

	
	// WR
	P1M0 |= (1<<2);
	P1M1 &= ~(1<<2); 
	
	// RD
	P1M0 |= (1<<3);
	P1M1 &= ~(1<<3);
	
	// RS/A0
	P1M0 |= (1<<1);
	P1M1 &= ~(1<<1);
	
	// CS
	P1M0 |= (1<<0);
	P1M1 &= ~(1<<0);
	
	// RST
	P3M0 |= (1<<7);
	P3M1 &= ~(1<<7);
	
	// INT (P36)

	// WAIT	(P55)
}


void LCD_CmdWrite(unsigned char cmd)
{
	_nop_();
	LCD_RS = 0;
	_nop_();
	LCD_CS = 0;
	_nop_();
	
	delay_us(2);
	P0 = cmd;
	
	_nop_();
	LCD_WR = 0;
	_nop_();
	LCD_WR = 1;
	_nop_();
	LCD_CS = 1;
	_nop_();
}

void LCD_DataWrite(unsigned char dat)
{
	_nop_();
	LCD_RS = 1;
	_nop_();
	LCD_CS = 0;
	_nop_();
	
	delay_us(2);
	P0 = dat;
	
	_nop_();
	LCD_WR = 0;
	_nop_();
	LCD_WR = 1;
	_nop_();
	LCD_CS = 1;
	_nop_();
}

void LCD_DataWrite_Pixel(unsigned int pixel)
{
	LCD_DataWrite(pixel);
	LCD_DataWrite(pixel >> 8);
}


unsigned char LCD_StatusRead(void)
{
	unsigned char temp;
	
  // 80808λ
	P0M0 = 0x00;
	P0M1 = 0xff; 
	
	_nop_();
	LCD_RS = 0;
	_nop_();
	LCD_CS = 0;
	_nop_();
	LCD_RD = 0;
	
	delay_us(1);
	temp = P0;
	
	_nop_();
	LCD_RD = 1;
	_nop_();
	LCD_CS = 1;
	_nop_();
	
	// 80808λ
	P0M0 = 0xff;
	P0M1 = 0x00;  
	
	return temp;
}


unsigned int LCD_DataRead(void)
{
	unsigned int temp;
	
  // 80808λ
	P0M0 = 0x00;
	P0M1 = 0xff; 

	_nop_();
	LCD_RS = 1;
	_nop_();
	LCD_CS = 0;
	_nop_();
	LCD_RD = 0;
	_nop_();
	
	delay_us(1);
	temp = P0;

	_nop_();
	LCD_RD = 1;
	_nop_();
	LCD_CS = 1;
	_nop_();
	 
	// 80808λ
	P0M0 = 0xff;
	P0M1 = 0x00; 
	
	return temp;
}

#endif


//-------------------------------------------------------------16λ8080-------------------------------------------------------

#if Peripheral_8080_16 

#include <intrins.h>


#define LCD_WR  P12
#define LCD_RD  P13
#define LCD_RS  P11
#define LCD_CS  P10

void Peripheral_8080_16_Init(void)
{
	// 8λ
	P0M0 = 0xff;
	P0M1 = 0x00; 
	// 8λ
	P2M0 = 0xff;
	P2M1 = 0x00; 
	
	// WR
	P1M0 |= (1<<2);
	P1M1 &= ~(1<<2); 
	
	// RD
	P1M0 |= (1<<3);
	P1M1 &= ~(1<<3);
	
	// RS/A0
	P1M0 |= (1<<1);
	P1M1 &= ~(1<<1);
	
	// CS
	P1M0 |= (1<<0);
	P1M1 &= ~(1<<0);
	
	// RST
	P3M0 |= (1<<7);
	P3M1 &= ~(1<<7);
	
	// INT (P36)

	// WAIT	(P55)
}


void LCD_CmdWrite(unsigned char cmd)
{
	_nop_();
	LCD_RS = 0;
	_nop_();
	LCD_CS = 0;
	_nop_();
	
	P0 = cmd;
	
	_nop_();
	LCD_WR = 0;
	_nop_();
	LCD_WR = 1;
	_nop_();
	LCD_CS = 1;
}

void LCD_DataWrite(unsigned char dat)
{
	_nop_();
	LCD_RS = 1;
	_nop_();
	LCD_CS = 0;
	_nop_();
	
	P0 = dat;
	
	_nop_();
	LCD_WR = 0;
	_nop_();
	LCD_WR = 1;
	_nop_();
	LCD_CS = 1;
}

void LCD_DataWrite_Pixel(unsigned int pixel)
{
	_nop_();
	LCD_RS = 1;
	_nop_();
	LCD_CS = 0;
	_nop_();
	
	P2 = (unsigned char)(pixel >> 8);
	P0 = (unsigned char)pixel;
	
	_nop_();
	LCD_WR = 0;
	_nop_();
	LCD_WR = 1;
	_nop_();
	LCD_CS = 1;
}


unsigned char LCD_StatusRead(void)
{
	unsigned char temp = 0;
	
  // 80808λ
	P0M0 = 0x00;
	P0M1 = 0xff; 
	// 80808λ
	P2M0 |= 0x00;
	P2M1 &= 0xff;
	
	_nop_();
	LCD_RS = 0;
	_nop_();
	LCD_CS = 0;
	_nop_();
	LCD_RD = 0;
	_nop_();
	
	delay_us(1);
	temp = P2;
	temp = (temp << 8) + P0;
	
	_nop_();
	LCD_RD = 1;
	_nop_();
	LCD_CS = 1;
	_nop_();
	
	// 80808λ
	P0M0 = 0xff;
	P0M1 = 0x00; 
	// 80808λ
	P2M0 |= 0xff;
	P2M1 &= 0x00;
	
	return temp;
}


unsigned int LCD_DataRead(void)
{
	unsigned int temp = 0;
	
  // 80808λ
	P0M0 = 0x00;
	P0M1 = 0xff; 
	// 80808λ
	P2M0 = 0x00;
	P2M1 = 0xff;
	
	_nop_();
	LCD_RS = 1;
	_nop_();
	LCD_CS = 0;
	_nop_();
	LCD_RD = 0;
	_nop_();
	
	delay_us(1);
	temp = P2;
	temp = (temp << 8) + P0;
	
	_nop_();
	LCD_RD = 1;
	_nop_();
	LCD_CS = 1;
	_nop_();
	
	// 80808λ
	P0M0 = 0xff;
	P0M1 = 0x00; 
	// 80808λ
	P2M0 = 0xff;
	P2M1 = 0x00;
	
	return temp;
}


#endif



//----------------------------------------------------------------IIC-------------------------------------------------------------
#if Peripheral_IIC

#define LT_ADDR 0xFC  // LCDΪ豸ӦIICַIICĵַLCDI2CA[0~5]I2CA[0~5]ƽ111111

#define SDA_OUT()      {P1M0 |= (1<<0);P1M1 &= ~(1<<0);}
#define SDA_IN()       {P1M0 &= ~(1<<0);P1M1 |= (1<<0);}
#define IIC_SCL_SET    P11 = 1;
#define IIC_SCL_RESET  P11 = 0;
#define IIC_SDA_SET    P10 = 1;
#define IIC_SDA_RESET  P10 = 0;
#define READ_SDA       P10
#define delay_nop()      {_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();_}

void IIC_Init(void)    
{
	
	P0M0 = 0x00;
	P0M1 = 0xff; 
	// 80808λ
	P1M0 = 0x00;
	P1M1 = 0xff;
	
   

	// RST
	P3M0 |= (1<<7);
	P3M1 &= ~(1<<7);
	
	//IIC_SCK
	P1M0 |= (1<<1);
    P1M1 &= ~(1<<1);
	
	//IIC_SDA
	P1M0 |= (1<<0);
    P1M1 &= ~(1<<0);
}

//IICʼź
void IIC_Start(void)
{
	SDA_OUT();     //sda
	IIC_SDA_SET;	  	  
	IIC_SCL_SET;
	delay_nop();
 	IIC_SDA_RESET;//START:when CLK is high,DATA change form high to low 
	delay_nop();
	IIC_SCL_RESET;//ǯסI2Cߣ׼ͻ 
}	  
//IICֹͣź
void IIC_Stop(void)
{
	SDA_OUT();//sda
	IIC_SCL_RESET;
	IIC_SDA_RESET;//STOP:when CLK is high DATA change form low to high
 	delay_nop();
	IIC_SCL_SET; 
	IIC_SDA_SET;//I2C߽ź
	delay_nop();							   	
}
//ȴӦźŵ
//ֵ1Ӧʧ
//        0Ӧɹ
unsigned char IIC_Wait_Ack(void)
{
	unsigned char ucErrTime=0;
	SDA_IN();      //SDAΪ  
	delay_nop();
	IIC_SDA_SET;delay_nop();  
	IIC_SCL_SET;delay_nop();	 
	while(READ_SDA)
	{
		ucErrTime++;
		if(ucErrTime>250)
		{
			IIC_Stop();
			return 1;
		}
	}
	IIC_SCL_RESET;//ʱ0 	   
	return 0;  
} 
//ACKӦ
void IIC_Ack(void)
{
	IIC_SCL_RESET;
	SDA_OUT();
	IIC_SDA_RESET;
	delay_nop();
	IIC_SCL_SET;
	delay_nop();
	IIC_SCL_RESET;
}
//ACKӦ		    
void IIC_NAck(void)
{
	IIC_SCL_RESET;
	SDA_OUT();
	IIC_SDA_SET;
	delay_nop();
	IIC_SCL_SET;
	delay_nop();
	IIC_SCL_RESET;
}					 	

//IICһֽ
//شӻӦ
//1Ӧ
//0Ӧ			  
void IIC_Send_Byte(unsigned char txd)
{                        
    unsigned char t;   
	SDA_OUT(); 	    
    IIC_SCL_RESET;//ʱӿʼݴ
    for(t=0;t<8;t++)
    {              
		if((txd&0x80)>>7)
		{IIC_SDA_SET;}
		else
		{IIC_SDA_RESET;}
		txd<<=1; 	  
		IIC_SCL_SET; 
		delay_nop();
		IIC_SCL_RESET;	
		delay_nop();

    }	 
} 	    
//1ֽڣack=1ʱACKack=0nACK   
unsigned char IIC_Read_Byte(unsigned char ack)
{
	unsigned char i,receive=0;
	SDA_IN();//SDAΪ
	_nop_();
	_nop_();
    for(i=0;i<8;i++ )
	{
        IIC_SCL_RESET; 
        delay_nop();
		IIC_SCL_SET;
        receive<<=1;
        if(READ_SDA)receive++;   
		delay_nop();	
    }					 
    if (!ack)
        IIC_NAck();//nACK
    else
        IIC_Ack(); //ACK   
    return receive;
}

unsigned char IIC_CmdWrite(unsigned char cmd)
{
	IIC_Start(); 
	IIC_Send_Byte(LT_ADDR);
	if(IIC_Wait_Ack())	//ȴӦ
	{
		IIC_Stop();		 
		return 1;		
	}
    IIC_Send_Byte(cmd);	
	if(IIC_Wait_Ack())	//ȴACK
	{
		IIC_Stop();	 
		return 1;		 
	}		 
    IIC_Stop();	 
	return 0 ;
}
unsigned char IIC_DataWrite(unsigned char dat)
{
	IIC_Start(); 
	IIC_Send_Byte(LT_ADDR|0x02);
	if(IIC_Wait_Ack())	//ȴӦ
	{
		IIC_Stop();		 
		return 1;		
	}
    IIC_Send_Byte(dat);	
	if(IIC_Wait_Ack())	//ȴACK
	{
		IIC_Stop();	 
		return 1;		 
	}		 
    IIC_Stop();	 
	return 0;
}

void IIC_DataWrite_Pixel(unsigned int dat)
{
	IIC_DataWrite(dat);
	IIC_DataWrite(dat>>8);
}

unsigned char IIC_StatusRead(void)
{
	unsigned char res;
    IIC_Start(); 
	IIC_Send_Byte(LT_ADDR|0x01);	
	IIC_Wait_Ack();			
	res=IIC_Read_Byte(0);
	IIC_NAck();
    IIC_Stop();		
	return res;		
}

unsigned char IIC_DataRead(void)
{
	unsigned char res;
    IIC_Start(); 
	IIC_Send_Byte(LT_ADDR|0x03);	
	IIC_Wait_Ack();			
	res=IIC_Read_Byte(0);
	IIC_NAck();
    IIC_Stop();		
	return res;
}

void LCD_CmdWrite(unsigned char cmd)
{
	IIC_CmdWrite(cmd);
}

void LCD_DataWrite(unsigned char dat)
{
	IIC_DataWrite(dat);
}

void LCD_DataWrite_Pixel(unsigned int dat)
{
	IIC_DataWrite_Pixel(dat);
}

unsigned char LCD_StatusRead(void)
{
    unsigned char temp = 0;
	temp = IIC_StatusRead();
	return temp;
}

unsigned int LCD_DataRead(void)
{
	unsigned int temp = 0;
	temp = IIC_DataRead();
	return temp;
}

#endif


void Interface_Init(void)
{
	#if Peripheral_HardwareSPI
		HardwareSPI_Init();
	#endif

	#if Peripheral_Software3SPI
		Software3SPI_Init();
	#endif

	#if Peripheral_Software4SPI
		Software4SPI_Init();
	#endif

		
	#if Peripheral_8080_8
		Peripheral_8080_8_Init();
	#endif
	
	#if Peripheral_8080_16
		Peripheral_8080_16_Init();
	#endif
	
	#if Peripheral_IIC
		IIC_Init();
	#endif
} 

